import React, {useState, useEffect} from 'react';
import ReactDOM from 'react-dom';
import { json, select, scaleLog, scalePoint, min, max } from 'd3';

//
// data
//
const jsonURL =
  'https://gist.githubusercontent.com/waseem-medhat/61e0416f6ed7ac64eb2872093c3bf8da/raw/7a86506fb4d3bdc581ce9f46eb42a6f78b17cec7/ww2020-11-11_metadata.json';

const useData = () => {
  const [data, setData] = useState();
  useEffect(() => {
    json(jsonURL).then(setData);
  }, []);
  return data;
};

//
// params
//
const width = 960;
const height = 500;
const margin = { top: 40, right: 60, bottom: 80, left: 30 };
const innerWidth = width - margin.right - margin.left;
const innerHeight = height - margin.top - margin.bottom;

const plotFrac = 0.6;
const statFrac = 1 - plotFrac;
const plotWidth = plotFrac * innerWidth;

//
// hover cards
//
const HoverCards = ({ data, yScale, innerWidth }) =>
  data.map((d, i) => (
    <rect
      className="card"
      y={yScale(d.STUDYID) - yScale.step() / 2}
      width={innerWidth}
      height={yScale.step()}
      rx="10"
      opacity={0}
    />
  ));

//
// heterogeneity bands
//
const HeterogeneityBands = ({
  data,
  xScale,
  innerHeight,
  innerWidth,
  plotWidth,
}) =>
  data.map((d) => (
    <rect
      className="hetereogeneity-band"
      transform={`translate(${innerWidth - plotWidth},0)`}
      x={xScale(d.OR_lo)}
      height={innerHeight}
      fill="#067"
      width={xScale(d.OR_hi) - xScale(d.OR_lo)}
      opacity={0.05 + 0.8 / data.length}
    />
  ));

//
// study summaries
//
const StudySummaries = ({ data, xScale, yScale, widthScale }) =>
  data.map((d) => (
    <g className="summary-mark">
      <line
        transform={`translate(0,${yScale(d.STUDYID)})`}
        x1={xScale(d.OR_lo)}
        x2={xScale(d.OR_hi)}
        stroke="#111"
        strokeWidth="2"
      />
      <rect
        x={xScale(d.OR) - widthScale(d.n) / 2}
        y={yScale(d.STUDYID) - widthScale(d.n) / 2}
        width={widthScale(d.n)}
        height={widthScale(d.n)}
      />
    </g>
  ));

//
// pooled summaries
//
const PooledSummaries = ({ data, xScale, yScale }) =>
  data.map((d) => {
    const polygonPath = `
    ${xScale(d.OR_lo)}, ${yScale(d.STUDYID)}
    ${xScale(d.OR)}, ${yScale(d.STUDYID) + yScale.step() / 3}
    ${xScale(d.OR_hi)}, ${yScale(d.STUDYID)}
    ${xScale(d.OR)}, ${yScale(d.STUDYID) - yScale.step() / 3}
`;
    return <polygon points={polygonPath} fill="#111" />;
  });

//
// axes
//
const Axes = ({ xScale, innerHeight, plotWidth, tickOffset = 20 }) => (
  <g className="axis">
    <line
      transform={`translate(${xScale(1)},0)`}
      y2={innerHeight}
      stroke="#111"
      strokeWidth="2"
    />

    <line
      transform={`translate(0,${innerHeight})`}
      x2={plotWidth}
      stroke="#111"
      strokeWidth="2"
    />

    {xScale.ticks(2).map((tick) => (
      <g
        className="tick"
        transform={`translate(${xScale(tick)},${innerHeight})`}
      >
        <text y={tickOffset} dy="0.32em" textAnchor="middle">
          {tick.toFixed(2)}
        </text>
        <line
          transform={`translate(0,${- tickOffset / 4})`}
          y2={tickOffset / 2}
          stroke="#111"
          strokeWidth="2"
        />
      </g>
    ))}
  </g>
);

//
// forest plot
//
const ForestPlot = ({
  data,
  xScale,
  yScale,
  widthScale,
  innerHeight,
  innerWidth,
  plotWidth,
}) => (
  <g transform={`translate(${innerWidth - plotWidth},0)`}>
    <Axes xScale={xScale} innerHeight={innerHeight} plotWidth={plotWidth} />

    <StudySummaries
      data={data.slice(0, data.length - 2)}
      xScale={xScale}
      yScale={yScale}
      widthScale={widthScale}
    />

    <PooledSummaries
      data={data.slice(data.length - 2)}
      xScale={xScale}
      yScale={yScale}
    />
  </g>
);

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleLog()
    .domain([min(data, (d) => d.OR_lo), 10])
    .range([0, plotWidth]);

  const yDomain = data.map((d) => d.STUDYID);
  yDomain.splice(data.length - 2, 0, ' ');

  const yScale = scalePoint()
    .domain(yDomain)
    .range([0, innerHeight])
    .padding(0.8);

  const widthScale = d3
    .scaleLinear()
    .domain(d3.extent(data, (d) => d.n))
    .range([10, 20]);

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <HeterogeneityBands
          data={data.slice(0, data.length - 2)}
          xScale={xScale}
          innerHeight={innerHeight}
          innerWidth={innerWidth}
          plotWidth={plotWidth}
        />

        {data.map((d) => (
          <g className='stat-area'>
            <text
              x={0.1 * statFrac * innerWidth}
              y={yScale(d.STUDYID)}
              dy="0.32em"
            >
              {d.STUDYID}
            </text>
            <text
              x={0.6 * statFrac * innerWidth}
              y={yScale(d.STUDYID)}
              dy="0.32em"
              textAnchor="middle"
            >
              {`${d.OR.toFixed(2)} (${d.OR_lo.toFixed(2)}, ${d.OR_hi.toFixed(
                2
              )})`}
            </text>
          </g>
        ))}

        <ForestPlot
          data={data}
          xScale={xScale}
          yScale={yScale}
          widthScale={widthScale}
          innerHeight={innerHeight}
          innerWidth={innerWidth}
          plotWidth={plotWidth}
        />

        <HoverCards data={data} yScale={yScale} innerWidth={innerWidth} />
      </g>
    </svg>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
